#/* Copyright 1998-2011 Research Foundation State University of New York */
#
#/* This file is part of QuB.                                            */
#
#/* QuB is free software; you can redistribute it and/or modify          */
#/* it under the terms of the GNU General Public License as published by */
#/* the Free Software Foundation, either version 3 of the License, or    */
#/* (at your option) any later version.                                  */
#
#/* QuB is distributed in the hope that it will be useful,               */
#/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
#/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
#/* GNU General Public License for more details.                         */
#
#/* You should have received a copy of the GNU General Public License,   */
#/* named LICENSE.txt, in the QuB program directory.  If not, see        */
#/* <http://www.gnu.org/licenses/>.                                      */

import sys
import os
import string
import re
import traceback

import QUB

#----------------------------------------------

def LoadCommands(path):
    for fname in os.listdir(path):
        if fname.endswith('.py') or fname.endswith('.pyc') or fname.endswith('.pyd'):
            try:
                # print 'importing %s' % fname
                exec 'import ' + fname[:string.rfind(fname,'.')]
            except Exception, msg:
                print '>>> import fname'
                traceback.print_exc()

#----------------------------------------------

class PromptField:
    def __init__(self, name, text, value='', choices=[]):
        self.name = name
        self.text = text
        self.value = value
        self.choices = choices

#-----------------------------------------------

def ParseTypedList(s, type):
    "e.g. '1, 2, 3-5 7' -> [1, 2, 3, 4, 5, 7]"

    if s == '':
        return []
    
    # replace \d+ ?- ?\d+ with \d+-\d+
    s = re.sub(r'(\d+) ?- ?(\d+)', r'\1-\2', s)
    # split along [, ]+
    tokens = re.split(r'[ ,\t]+', s)
    # foreach token:  if \d+ append int; elif \d+-\d+ append ints
    result = []
    for tok in tokens:
        match = re.search(r'(\d+)-(\d+)', tok)
        if match:
            a, b = type(match.group(1)), type(match.group(2))
            if a <= b:
                for x in xrange(a, b+1):
                    result.append(x)
            else:
                for x in xrange(a, b-1, -1):
                    result.append(x)
        else:
            result.append( type(tok) )
    return result

ParseIntList = lambda s: ParseTypedList(s, int)

#-----------------------------------------------

import numpy
import ctypes
from ctypes import c_double, c_int, c_long, c_size_t, py_object, POINTER, byref, cast, sizeof
c_int_p = POINTER(c_int)
c_long_p = POINTER(c_long)
c_size_t_p = POINTER(c_size_t)
c_double_p = POINTER(c_double)

pybuf = ctypes.pythonapi.PyBuffer_FromReadWriteMemory
pybuf.restype = py_object
pyobj_asreadbuf = ctypes.pythonapi.PyObject_AsReadBuffer
pyobj_asreadbuf.argtypes = (py_object, c_long_p, c_size_t_p)
def buffer_to_pointer(buf):
    ptr = c_long()
    plen = c_size_t()
    pyobj_asreadbuf(buf, byref(ptr), byref(plen))
    return cast(ptr.value, c_int_p)

#import scipy.linalg
#def Eigen_Scipy(n, a_, wr_, wi_, v_, v_inv_):
#    try:
#        a = numpy.frombuffer(pybuf(cast(a_, c_double_p), n*n*sizeof(c_double)), dtype='float64', count=n*n).reshape((n,n))
#        v = numpy.frombuffer(pybuf(cast(v_, c_double_p), n*n*sizeof(c_double)), dtype='float64', count=n*n).reshape((n,n))
#        v_inv = numpy.frombuffer(pybuf(cast(v_inv_, c_double_p), n*n*sizeof(c_double)), dtype='float64', count=n*n).reshape((n,n))
#        wr_ = cast(wr_, c_double_p)
#        wi_ = cast(wi_, c_double_p)
#        if n == 1:
#            wr_[0] = a[0,0]
#            wi_[0] = 0.0
#            v[0,0] = v_inv[0,0] = 1.0
#        else:
#            eig, vecs = scipy.linalg.eig(a)
#            for i in xrange(n):
#                wr_[i] = numpy.real(eig[i])
#                wi_[i] = numpy.imag(eig[i])
#                for j in xrange(n):
#                    v[i,j] = vecs[i,j]
#            v_inv[:,:] = numpy.matrix(vecs).I
#        return 0
#    except:
#        QUB.Report(traceback.format_exc())
#        return -1


def StrToArrayOfInt(s, a_, maxn):
    a = numpy.frombuffer(pybuf(cast(a_, c_int_p), maxn*sizeof(c_int)), dtype='int32', count=maxn).reshape((maxn,))
    ii = [int(x) for x in re.split(r"[ ,\t\r\n]+", s)]
    n = min(len(ii), maxn)
    a[:n] = ii[:n]
    return n
